var geocoder;
var map;

/**
 * Performs initialisation of a Google Map setting a default position based on a longitude and lattitude setting. 
 * <p/>
 * The map is placed into the group that is specified as a parameter.
 * Further functions allow the position on the map to be altered, and a pin placed to mark the spot.
 * <p/>
 * @author  Hyfinity Limited 
 *
 * @param mapCanvas {Group_Name} identifies the group container on the page for the google map content e.g. map_canvas.
 * You can pass the name of the group or the group component object. 
*/
function initializeMap(objMapCanvas)
{
    var canvas;
    if (typeof(objMapCanvas) == 'string')
    {	    
        canvas = document.getElementById(objMapCanvas);
    }
    else
    {
	canvas = objMapCanvas;	    
    }   
    // return if mandatory paramters are missing.
    if (canvas == null)
    {
    	return;
    }


    if (canvas != null)
    {
        geocoder = new google.maps.Geocoder();
        var latlng = new google.maps.LatLng(37.641078, -122.055843);
        var myOptions = {
            zoom : 14,
            center : latlng,
            mapTypeId : google.maps.MapTypeId.ROADMAP,
            streetViewControl : true
        }
        map = new google.maps.Map(canvas, myOptions);
    }
    google.maps.event.addDomListener(window, 'load', initializeMap);
    
};

/**
 * Performs GeoCoding on the supplied address field to identify a position on the map to place a pin. 
 * <p/>
 * It will also show a information window with the details if the pin is clicked.  
 * If the pinImage is supplied, it must be the location of the image in the webapps directory e.g. images/blue_MarkerO.png
 * <p/>
 * Alternatively, it can be a full remote URL such as: 
 * <p/>
 * <ul> 
 * <li> Google Red   Pin - pinImage = 'http://maps.google.com/mapfiles/ms/icons/red-dot.png';</li>
 * <li> Google Green Pin - pinImage = 'http://maps.google.com/mapfiles/ms/icons/green-dot.png';</li>
 * <li> Google Blue  Pin - pinImage = 'http://maps.google.com/mapfiles/ms/icons/blue-dot.png';</li>
 * </ul>
 * <p/>
 * @author  Hyfinity Limited 
 *
 * @param   address      {Field_Name} contains the address text for geo-coding. You can pass the name of the group or the group component object.  
 * @param   pinImage     {string}        (optional)  contains the location URI of image file.   
*/  
function codeAddress(objAddress,pinImage)
{
    var address;
    if (typeof(objAddress) == 'string')
    {	    
        address = document.getElementById(objAddress);
    }
    else
    {
	address = objAddress;	    
    }  
    // return if mandatory parameters are missing.
    if (address == null)   
    {
    	return;
    }
    
    
    if (document.getElementById(objAddress) != null)
    {
        var address = document.getElementById(objAddress).value;

        if(geocoder)
        {
            geocoder.geocode( {'address': address}, function(results, status) {
                    if(status == google.maps.GeocoderStatus.OK)
                    {
                        map.setCenter(results[0].geometry.location);

                        if (!pinImage || pinImage.length == 0)
                        {      
                            pinImage = new google.maps.MarkerImage('http://maps.google.com/mapfiles/ms/icons/red-dot.png');
                        }       
                        var pinShadowImage = new google.maps.MarkerImage('http://chart.apis.google.com/chart?chst=d_map_pin_shadow',
                                             new google.maps.Size(40, 37),
                                             new google.maps.Point(0, 0),
                                             new google.maps.Point(12, 35));   
            
                        var marker = new google.maps.Marker({
                                map: map,
                                position: results [ 0 ].geometry.location,
                                title: address,
                                icon: pinImage,
                                shadow: pinShadowImage                                 
                            });
           
                        var infowindow = new google.maps.InfoWindow(
                        {
                                content: address           
                         });
                            
                        google.maps.event.addListener(marker, 'click', function() 
                        {  
                                infowindow.open(map,marker);
                        });                              
                    }
                    else
                    {
                        alert("Geocode was not successful for the following reason: " + status);
                    }
                });
        }
    }
}

/**
 * Performs GeoCoding for the current location of the device/computer and places a pin on the map.
 * <p/>
 * It will also show a information window with the details if the pin is clicked. 
 * The current location can only be identified if the user accepts a question.  
 * If the pinImage is supplied, it must be the location of the image in the webapps directory e.g. images/blue_MarkerO.png.
 * <p/>
 * Alternatively, it can be a full remote URL such as: 
 * <p/>
 * <ul> 
 * <li> Google Red   Pin - pinImage = 'http://maps.google.com/mapfiles/ms/icons/red-dot.png';</li>
 * <li> Google Green Pin - pinImage = 'http://maps.google.com/mapfiles/ms/icons/green-dot.png';</li>
 * <li> Google Blue  Pin - pinImage = 'http://maps.google.com/mapfiles/ms/icons/blue-dot.png';</li>
 * </ul>
 * <p/>
 * @author  Hyfinity Limited 
 *
 * @param   pinTitle   {string} (optional) contains a title to be displayed if the pin is hovered over, or clicked on  
 * @param   pinImage   {string} (optional) contains the location URI of image file 
*/ 
function codeCurrentLocation(pinTitle,pinImage)
{
    // Try HTML5 geolocation
        if(navigator.geolocation) 
        {
          navigator.geolocation.getCurrentPosition(function(position) 
          {
            var pos = new google.maps.LatLng(position.coords.latitude,
                                             position.coords.longitude);

            if (!pinTitle || pinTitle.length == 0)
            {      
                pinTitle = 'Your Current Location';
            }    
            if (!pinImage || pinImage.length == 0)
            {      
                pinImage = new google.maps.MarkerImage('http://maps.google.com/mapfiles/ms/icons/blue-dot.png');
            }       
                          
            var pinShadowImage = new google.maps.MarkerImage('http://chart.apis.google.com/chart?chst=d_map_pin_shadow',
                                 new google.maps.Size(40, 37),
                                 new google.maps.Point(0, 0),
                                 new google.maps.Point(12, 35));           
      
            var marker = new google.maps.Marker(
            {
                    map: map,
                    position: pos,
                    title: pinTitle,
                    icon: pinImage,
                    shadow: pinShadowImage                    
            });
           
           var infowindow = new google.maps.InfoWindow(
           {
                    content: pinTitle           
            });
                
            google.maps.event.addListener(marker, 'click', function() 
            {  
                infowindow.open(map,marker);
            });  

            map.setCenter(pos);
          }, function() {
            handleNoGeolocation(true);
          });
        } else {
          // Browser doesn't support Geolocation
          handleNoGeolocation(false);
        }

      function handleNoGeolocation(errorFlag) 
      {
        if (errorFlag) 
        {
          var content = 'Error: The Geolocation service failed.';
        } else 
        {
          var content = 'Error: Your browser doesn\'t support geolocation.';
        }

        var options = 
        {
          map: map,
          position: new google.maps.LatLng(37.641078, -122.055843),
          content: content
        };

        var infowindow = new google.maps.InfoWindow(options);
        map.setCenter(options.position);
      }
}